import pandas as pd
from pandashape import Columns
from pandashape.describers.GeneralDescriber import GeneralDescriber
from pandashape.internal.TransformerExecutor import TransformerExecutor


class PandaShaper:
    def __init__(self, df, inplace=False):
        assert(isinstance(df, pd.DataFrame))

        # why does this cause an evaluation of the truth of df?
        # self.df = df if inplace else df.copy(df)
        self.df = df
        if not inplace:
            self.df = df.copy()

    def describe(self, columnDefinitions=None):
        if columnDefinitions is None:
            describer = GeneralDescriber(self.df)
            return describer.describe(self.df)

    def transform(self, columnDefinitions):
        executor = TransformerExecutor()
        newColumns = executor.transform(self.df, columnDefinitions)
        print('new columns!', newColumns)



from .GenericDescriber import GenericDescriber


class GeneralDescriber(GenericDescriber):
    def __init__(self, df):
        super().__init__(df)

    def describe(self, df):
        messages = [f"Shape: {df.shape}"]

        columns_with_nulls = df.columns[df.isna().any()].array
        if len(columns_with_nulls) > 0:
            messages.append(
                f"Columns with one or more null values: {columns_with_nulls}"
            )

        object_typed_columns = df.select_dtypes(include='object').columns.array
        if len(object_typed_columns) > 0:
            messages.append(
                f"Columns of type \"object\" (may need label encoding): ${object_typed_columns}"
            )

        return messages



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pandashape",
    version="0.0.1",
    author="Ben Stein",
    author_email="ben.s.stein@gmail.com",
    description="A package designed to simplify data preprocessing for use with Pandas",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jammerware/panda-shape.git",
    packages=setuptools.find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests", "*.internal"]),
    install_requires=[
        "pandas",
        "numpy"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from pandas import DataFrame, Series
import pandas as pd
from pandashape.transformers.GenericTransformer import GenericTransformer


class MassLabelEncoder(GenericTransformer):
    def __init__(self, label_encoding_breakpoint=0):
        self.label_encoding_breakpoint = label_encoding_breakpoint

    def transform(self, df):
        assert(isinstance(df, DataFrame))
        categorical_features = (
            df
            .select_dtypes(include='object')
            .columns
            .array
            .to_numpy()
            .tolist()
        )

        # print(categorical_features)

    def transformSeries(self, series):
        unique_value_count = len(series.astype('category').cat.codes)

        if unique_value_count >= self.label_encoding_breakpoint:
            return self.__labelEncode(series)
        else:
            return self.__oneHotEncode(series)

    def __labelEncode(self, series):
        return pd.Series(name=series.name, data=series.astype('category').cat.codes)

    def __oneHotEncode(self, series):
        return pd.get_dummies(series)
